@@ -111,7 +111,7 @@ int main(int argc, char **argv)
     {
         if(!(font = create_empty_font())) exit_gracefully();
     }
-    refresh_view_status_msg(font_file_name, font);
+    //refresh_view_status_msg(font_file_name, font);
   
     /*
      * TODO: We should use this information to add
@@ -121,7 +121,7 @@ int main(int argc, char **argv)
     else big_endian = 0;
   
     int i;
-    int ch;
+    int ch = 0;
     char *abt;
     while(!end) 
     {	//infinite program loop//
