@@ -545,14 +545,28 @@ skip:
 	  }
 	  else if(sel == 3)
 	  {
-            /* we deliberately avoid changing BDF font length for now.
-             * TODO: fix this!
-             */
-            if(font->version == VER_BDF) break;
-	    if(font->length < 256) font->length = 256;
+            if(font->version == VER_BDF)
+            {
+                char s[10];
+                sprintf(s, "%u", font->length);
+                inputBoxI("Enter new glyph count:", s, " Glyph count ");
+                if(strlen(input) == 0)
+                {
+                    refresh_metrics_window(font, x, y, sel);
+                    break;
+                }
+                int l = atoi(input);
+                if(l <= 0)
+                {
+                    msgBox("Invalid glyph count.", OK, ERROR);
+                }
+                else font->length = l;
+            }
+	    else if(font->length < 256) font->length = 256;
 	    else if(font->length < 512) font->length = 512;
 	    else font->length = 256;
-	    struct module_s *mod = get_module_by_name(font->version < 3 ? "psf" :
+	    
+            struct module_s *mod = get_module_by_name(font->version < 3 ? "psf" :
                                                 get_version_str(font->version));
 	    if(font->length > mod->max_length) font->length = mod->max_length;
 	    //if(font->version == VER_CP && font->length != 256) font->version = VER_PSF2;
