@@ -27,38 +27,38 @@
 struct bdf_keyword_s bdf_keywords[] =
 {
   /* Global keywords */
-  { "STARTFONT",       BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, { NUMBER } },
-  { "COMMENT",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "CONTENTVERSION",  BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "FONT",            BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "SIZE",            BDF_REQ_YES, BDF_GLOBAL_SCOPE, 3, { NUMBER, NUMBER, NUMBER } },
-  { "FONTBOUNDINGBOX", BDF_REQ_YES, BDF_GLOBAL_SCOPE, 4, { INTEGER, INTEGER, INTEGER, INTEGER } },
-  { "METRICSSET",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "SWIDTH",          BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
-  { "DWIDTH",          BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
-  { "SWIDTH1",         BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
-  { "DWIDTH1",         BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
-  { "VVECTOR",         BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
-  { "STARTPROPERTIES", BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "ENDPROPERTIES",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 0 },
-  { "CHARS",           BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "ENDFONT",         BDF_REQ_YES, BDF_GLOBAL_SCOPE, 0 },
-  { "COPYRIGHT",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "FOUNDRY",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "FAMILY_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "WEIGHT_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "SLANT",           BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "SETWIDTH_NAME",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "FONT_VERSION",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "FONT_TYPE",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "PIXEL_SIZE",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "POINT_SIZE",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "RESOLUTION_X",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "RESOLUTION_Y",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "SPACING",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "AVERAGE_WIDTH",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "CHARSET_REGISTRY",BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
-  { "CHARSET_ENCODING",BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "STARTFONT",          BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, { NUMBER  } },
+  { "COMMENT",            BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "CONTENTVERSION",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "FONT",               BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "SIZE",               BDF_REQ_YES, BDF_GLOBAL_SCOPE, 3, { NUMBER, NUMBER, NUMBER } },
+  { "FONTBOUNDINGBOX",    BDF_REQ_YES, BDF_GLOBAL_SCOPE, 4, { INTEGER, INTEGER, INTEGER, INTEGER } },
+  { "METRICSSET",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "SWIDTH",             BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "DWIDTH",             BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "SWIDTH1",            BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "DWIDTH1",            BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "VVECTOR",            BDF_REQ_NO,  BDF_ANY_SCOPE,    2, { NUMBER, NUMBER } },
+  { "STARTPROPERTIES",    BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "ENDPROPERTIES",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 0 },
+  { "CHARS",              BDF_REQ_YES, BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "ENDFONT",            BDF_REQ_YES, BDF_GLOBAL_SCOPE, 0 },
+  { "COPYRIGHT",          BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "FOUNDRY",            BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "FAMILY_NAME",        BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "WEIGHT_NAME",        BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "SLANT",              BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "SETWIDTH_NAME",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "FONT_VERSION",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "FONT_TYPE",          BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "PIXEL_SIZE",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "POINT_SIZE",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "RESOLUTION_X",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "RESOLUTION_Y",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "SPACING",            BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "AVERAGE_WIDTH",      BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
+  { "CHARSET_REGISTRY",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
+  { "CHARSET_ENCODING",   BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
   { "UNDERLINE_POSITION", BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
   { "UNDERLINE_THICKNESS",BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
   { "CAP_HEIGHT",         BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
@@ -66,13 +66,13 @@ struct bdf_keyword_s bdf_keywords[] =
   { "FONT_ASCENT",        BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
   { "FONT_DESCENT",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
   { "DEFAULT_CHAR",       BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { INTEGER } },
-  { "ADD_STYLE_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING } },
+  { "ADD_STYLE_NAME",     BDF_REQ_NO,  BDF_GLOBAL_SCOPE, 1, { STRING  } },
   /* Glyph keywords */
-  { "STARTCHAR",       BDF_REQ_YES, BDF_GLYPH_SCOPE,  1, { STRING } },
-  { "ENCODING",        BDF_REQ_YES, BDF_GLYPH_SCOPE,  1, { INTEGER } },
-  { "BBX",             BDF_REQ_YES, BDF_GLYPH_SCOPE,  4, { INTEGER, INTEGER, INTEGER, INTEGER } },
-  { "BITMAP",          BDF_REQ_YES, BDF_GLYPH_SCOPE,  0 },
-  { "ENDCHAR",         BDF_REQ_YES, BDF_GLYPH_SCOPE,  0 },
+  { "STARTCHAR",          BDF_REQ_YES, BDF_GLYPH_SCOPE,  1, { STRING  } },
+  { "ENCODING",           BDF_REQ_YES, BDF_GLYPH_SCOPE,  1, { INTEGER } },
+  { "BBX",                BDF_REQ_YES, BDF_GLYPH_SCOPE,  4, { INTEGER, INTEGER, INTEGER, INTEGER } },
+  { "BITMAP",             BDF_REQ_YES, BDF_GLYPH_SCOPE,  0 },
+  { "ENDCHAR",            BDF_REQ_YES, BDF_GLYPH_SCOPE,  0 },
 };
 
 /* we will use this bitmap to check that all the required words
@@ -149,14 +149,23 @@ static inline unsigned int get_hex(char *s, int nibbles)
   {
     unsigned char h = 0;
     if(*s >= 'a' && *s <= 'f') h = *s-'a'+10;
-    if(*s >= 'A' && *s <= 'F') h = *s-'A'+10;
-    if(*s >= '0' && *s <= '9') h = *s-'0';
+    else if(*s >= 'A' && *s <= 'F') h = *s-'A'+10;
+    else if(*s >= '0' && *s <= '9') h = *s-'0';
+    else break;
     res = (res << 4) | h;
     s++;
   }
   return res;
 }
 
+static inline int ishexdigit(char c)
+{
+    if(c >= 'a' && c <= 'f') return 1;
+    if(c >= 'A' && c <= 'F') return 1;
+    if(c >= '0' && c <= '9') return 1;
+    return 0;
+}
+
 
 extern unsigned short default_unicode_table[]; /* psf.c */
 
@@ -275,11 +284,14 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	if(!font) goto memory_error;
 	memset((void *)font, 0, sizeof(struct font_s));
 	unsigned char *data = (unsigned char *)NULL;
+        /*
 	font->unicode_info_size = (512 * 4);
 	font->unicode_info = (unsigned char *)malloc(font->unicode_info_size);
 	if(!font->unicode_info) goto memory_error;
 	memset((void *)font->unicode_info, 0, font->unicode_info_size);
 	unsigned short *unicode_info = (unsigned short *)font->unicode_info;
+        */
+        unsigned short *unicode_info = NULL;
 	font->has_unicode_table = 1;
 	font->utf_version = 1;
 	font->has_metadata = 1;
@@ -296,20 +308,23 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	if(!s) goto file_read_error;
 	keyword_bitmap = 0;
 	/* there is no signature. just make sure the first line is legit. */
-	if(strncmp(s, "STARTFONT", 9)) goto corrupt_file;
-	/* check the version. we support ver 2.1 */
+	/*
+        if(strncmp(s, "STARTFONT", 9)) goto corrupt_file;
+	// check the version. we support ver 2.1 //
 	if(strcmp(s, "STARTFONT 2.1"))
 	{
 	  status_error("BDF versions apart from 2.1 are not supported");
 	  goto go;
 	}
+	*/
 	char scope = BDF_GLOBAL_SCOPE;
 	int w = 0, h = 0;
 	int globalw = 0, globalh = 0;
         char globaln = 0;
 	int xoff = 0, yoff = 0;
 	int globalxoff = 0, globalyoff = 0;
-	int i, data_index = 0, chars = 0;
+	int i, chars = 0;
+        unsigned int data_index = 0;
 	
 	do
 	{
@@ -326,251 +341,301 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	  /* mark the keyword as found in our bitmap */
 	  keyword_bitmap |= (1 << index);
 	  /* now check the keyword */
-	  if(index == STARTPROPERTIES_KEYWORD)
-	  {
-		scope = BDF_PROPERTIES_SCOPE;
-	  }
-	  else if(index == ENDPROPERTIES_KEYWORD)
-	  {
-		scope = BDF_GLOBAL_SCOPE;
-	  }
-	  else if(index == FONT_KEYWORD)
+          switch(index)
           {
-                skip_spaces(&s);
-                int z = 0;
-                while(*s != '\0') buf[z++] = *s++;
-		save_metadata_str(font, METADATA_FONT, buf); //get_string(&s));
-          }
-	  else if(index == COPYRIGHT_KEYWORD)
-		save_metadata_str(font, METADATA_COPYRIGHT, get_string(&s));
-	  else if(index == FONT_VERSION_KEYWORD)
-		save_metadata_str(font, METADATA_FONT_VERSION, get_string(&s));
-	  else if(index == FONT_TYPE_KEYWORD)
-		save_metadata_str(font, METADATA_FONT_TYPE, get_string(&s));
-	  else if(index == FOUNDRY_KEYWORD)
-		save_metadata_str(font, METADATA_FOUNDRY, get_string(&s));
-	  else if(index == FAMILY_NAME_KEYWORD)
-		save_metadata_str(font, METADATA_FAMILY_NAME, get_string(&s));
-	  else if(index == WEIGHT_NAME_KEYWORD)
-		save_metadata_str(font, METADATA_WEIGHT_NAME, get_string(&s));
-	  else if(index == SLANT_KEYWORD)
-		save_metadata_str(font, METADATA_SLANT, get_string(&s));
-	  else if(index == SETWIDTH_NAME_KEYWORD)
-		save_metadata_str(font, METADATA_SETWIDTH_NAME, get_string(&s));
-	  else if(index == PIXEL_SIZE_KEYWORD)
-		metadata[METADATA_PIXEL_SIZE].value = get_integer(&s);
-	  else if(index == POINT_SIZE_KEYWORD)
-		metadata[METADATA_POINT_SIZE].value = get_integer(&s);
-	  else if(index == RESOLUTION_X_KEYWORD)
-		metadata[METADATA_RESOLUTION_X].value = get_integer(&s);
-	  else if(index == RESOLUTION_Y_KEYWORD)
-		metadata[METADATA_RESOLUTION_Y].value = get_integer(&s);
-	  else if(index == SPACING_KEYWORD)
-		save_metadata_str(font, METADATA_SPACING, get_string(&s));
-	  else if(index == AVERAGE_WIDTH_KEYWORD)
-		metadata[METADATA_AVERAGE_WIDTH].value = get_integer(&s);
-	  else if(index == CHARSET_REGISTRY_KEYWORD)
-		save_metadata_str(font, METADATA_CHARSET_REGISTRY, get_string(&s));
-	  else if(index == CHARSET_ENCODING_KEYWORD)
-		save_metadata_str(font, METADATA_CHARSET_ENCODING, get_string(&s));
-	  else if(index == ADD_STYLE_NAME_KEYWORD)
-		save_metadata_str(font, METADATA_ADD_STYLE_NAME, get_string(&s));
-	  else if(index == UNDERLINE_POSITION_KEYWORD)
-		metadata[METADATA_UNDERLINE_POSITION].value = get_integer(&s);
-	  else if(index == UNDERLINE_THICKNESS_KEYWORD)
-		metadata[METADATA_UNDERLINE_THICKNESS].value = get_integer(&s);
-	  else if(index == CAP_HEIGHT_KEYWORD)
-		metadata[METADATA_CAP_HEIGHT].value = get_integer(&s);
-	  else if(index == X_HEIGHT_KEYWORD)
-		metadata[METADATA_X_HEIGHT].value = get_integer(&s);
-	  else if(index == FONT_ASCENT_KEYWORD)
-		metadata[METADATA_FONT_ASCENT].value = get_integer(&s);
-	  else if(index == FONT_DESCENT_KEYWORD)
-		metadata[METADATA_FONT_DESCENT].value = get_integer(&s);
-	  else if(index == DEFAULT_CHAR_KEYWORD)
-		metadata[METADATA_DEFAULT_CHAR].value = get_integer(&s);
-	  else if(index == FONTBOUNDINGBOX_KEYWORD)
-	  {
-		if(scope != BDF_GLOBAL_SCOPE) goto corrupt_file;
-		globalw = get_integer(&s);
-		globalh = get_integer(&s);
-		globalxoff = get_integer(&s);
-		globalyoff = get_integer(&s);
-		font->width = globalw;
-		font->height = globalh;
-                globaln = ((globalw+7)/8)*2;
-		font->charsize = font->height*((font->width+7)/8);
-		metadata[METADATA_FONTBOUNDINGBOX_X].value = globalw;
-		metadata[METADATA_FONTBOUNDINGBOX_Y].value = globalh;
-		metadata[METADATA_FONTBOUNDINGBOX_XOFF].value = globalxoff;
-		metadata[METADATA_FONTBOUNDINGBOX_YOFF].value = globalyoff;
-	  }
-	  else if(index == CHARS_KEYWORD)
-	  {
-		scope = BDF_CHARS_SCOPE;
-		font->length = get_integer(&s);
-		if(font->length > 512)
-		{
-		  status_error("We don't support font length > 512 chars. Sorry!");
-		  //msgBox("We don't support font length > 512 chars. Sorry!", OK, ERROR);
-		  goto go;
-		}
-		data = (unsigned char *)malloc(font->charsize*font->length);
-		if(!data) goto memory_error;
-		memset((void *)data, 0, font->charsize*font->length);
-		font->data = data;
-		font->data_size = font->charsize*font->length;
-                font->unicode_info_size = (font->length * 4);
-                /*
-                char_info = (struct char_info_s *)malloc(font->length*sizeof(struct char_info_s));
-                if(!char_info) goto memory_error;
-                font->char_info = (void *)char_info;
-                font->char_info_size = font->length*sizeof(struct char_info_s);
-                */
-                if(!create_char_info(font)) goto memory_error;
-                char_info = font->char_info;
-	  }
-	  else if(index == STARTCHAR_KEYWORD)
-	  {
-		scope = BDF_GLYPH_SCOPE;
-		skip_spaces(&s);
-		/* check if it is Unicode-encoded */
-                unsigned int chr = 0;
-		if(s[0] != 'U' || s[1] != '+')
-		{
-		  if(*s >= '0' && *s <= '9') chr = get_hex(s, 4);
-		  else
+              case STARTFONT_KEYWORD:
+                  skip_spaces(&s);
+                  /* check the version. we support ver 2.1 */
+                  if(strcmp(s, "2.1"))
                   {
-                      chr = postscript_to_unicode(s);
-                      if(!chr)
-                      {
-                          //chr = codepoint_to_unicode(s);
-                          if(strlen(s) == 4) chr = get_hex(s, 4);
-                      }
-                  }
-                  if(!chr)
-                  {
-                      msgBox("Char name not in Unicode, Adobe\n"
-                                    "Standard Encoding, or Postscript char\n"
-                                    "name. I Can't read this encoding!", OK, ERROR);
+                      status_error("BDF versions apart from 2.1 are not supported");
                       goto go;
                   }
-		}
-		else
-                {
-                    s += 2;
-                    chr = get_hex(s, 4);
-                }
-		unicode_info[chars*2] = (unsigned short)chr;
-		unicode_info[(chars*2)+1] = 0xFFFF;
-	  }
-	  else if(index == ENDCHAR_KEYWORD)
-	  {
-		scope = BDF_CHARS_SCOPE;
-		chars++;
-	  }
-	  else if(index == ENDFONT_KEYWORD)
-	  {
-		scope = BDF_GLOBAL_SCOPE;
-		break;
-	  }
-	  else if(index == BBX_KEYWORD)
-	  {
-		if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
-		w = get_integer(&s);
-		h = get_integer(&s);
-		xoff = get_integer(&s);
-		yoff = get_integer(&s);
-                if(font->char_info)
-                {
-                    char_info[chars].BBw = w;
-                    char_info[chars].BBh = h;
-                    char_info[chars].BBXoff = xoff;
-                    char_info[chars].BBYoff = yoff;
-                }
-	  }
-	  else if(index == SWIDTH_KEYWORD)
-          {
-		if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
-                if(font->char_info)
-                {
-                    int swx1 = get_integer(&s);
-                    int swy1 = get_integer(&s);
-                    char_info[chars].swidthX = swx1;
-                    char_info[chars].swidthY = swy1;
-                }
-          }
-	  else if(index == DWIDTH_KEYWORD)
-          {
-		if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
-                if(font->char_info)
-                {
-                    int dwx1 = get_integer(&s);
-                    int dwy1 = get_integer(&s);
-                    char_info[chars].dwidthX = dwx1;
-                    char_info[chars].dwidthY = dwy1;
-                }
-          }
-	  else if(index == ENCODING_KEYWORD)
-          {
-		if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
-                if(font->char_info)
-                {
-                    int encoding = get_integer(&s);
-                    char_info[chars].encoding = encoding;
-                }
-          }
-	  else if(index == BITMAP_KEYWORD)
-	  {
-		if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
-		if(!font->data) goto corrupt_file;
-		if(!w || !h)
-		{
-		  if(!globalw || !globalh)
-			goto corrupt_file;
-		  //w = globalw; h = globalh;
-		}
-		i = h;
-		while(i && (s = strtok(NULL, "\n")))
-		{
-		  /* how many nibbles do we have per line? */
-		  char n = ((w+7)/8)*2;
+                  break;
+              case STARTPROPERTIES_KEYWORD:
+                  scope = BDF_PROPERTIES_SCOPE;
+                  break;
+              case ENDPROPERTIES_KEYWORD:
+                  scope = BDF_GLOBAL_SCOPE;
+                  break;
+              case FONT_KEYWORD:
+                  skip_spaces(&s);
+                  int z = 0;
+                  while(*s != '\0') buf[z++] = *s++;
+		  save_metadata_str(font, METADATA_FONT, buf); //get_string(&s));
+                  break;
+              case COPYRIGHT_KEYWORD:
+		  save_metadata_str(font, METADATA_COPYRIGHT, get_string(&s));
+                  break;
+              case FONT_VERSION_KEYWORD:
+		  save_metadata_str(font, METADATA_FONT_VERSION, get_string(&s));
+                  break;
+              case FONT_TYPE_KEYWORD:
+		  save_metadata_str(font, METADATA_FONT_TYPE, get_string(&s));
+                  break;
+              case FOUNDRY_KEYWORD:
+                  save_metadata_str(font, METADATA_FOUNDRY, get_string(&s));
+                  break;
+              case FAMILY_NAME_KEYWORD:
+                  save_metadata_str(font, METADATA_FAMILY_NAME, get_string(&s));
+                  break;
+              case WEIGHT_NAME_KEYWORD:
+                  save_metadata_str(font, METADATA_WEIGHT_NAME, get_string(&s));
+                  break;
+              case SLANT_KEYWORD:
+                  save_metadata_str(font, METADATA_SLANT, get_string(&s));
+                  break;
+              case SETWIDTH_NAME_KEYWORD:
+                  save_metadata_str(font, METADATA_SETWIDTH_NAME, get_string(&s));
+                  break;
+              case PIXEL_SIZE_KEYWORD:
+                  metadata[METADATA_PIXEL_SIZE].value = get_integer(&s);
+                  break;
+              case POINT_SIZE_KEYWORD:
+                  metadata[METADATA_POINT_SIZE].value = get_integer(&s);
+                  break;
+              case RESOLUTION_X_KEYWORD:
+                  metadata[METADATA_RESOLUTION_X].value = get_integer(&s);
+                  break;
+              case RESOLUTION_Y_KEYWORD:
+                  metadata[METADATA_RESOLUTION_Y].value = get_integer(&s);
+                  break;
+              case SPACING_KEYWORD:
+                  save_metadata_str(font, METADATA_SPACING, get_string(&s));
+                  break;
+              case AVERAGE_WIDTH_KEYWORD:
+                  metadata[METADATA_AVERAGE_WIDTH].value = get_integer(&s);
+                  break;
+              case CHARSET_REGISTRY_KEYWORD:
+                  save_metadata_str(font, METADATA_CHARSET_REGISTRY, get_string(&s));
+                  break;
+              case CHARSET_ENCODING_KEYWORD:
+                  save_metadata_str(font, METADATA_CHARSET_ENCODING, get_string(&s));
+                  break;
+              case ADD_STYLE_NAME_KEYWORD:
+                  save_metadata_str(font, METADATA_ADD_STYLE_NAME, get_string(&s));
+                  break;
+              case UNDERLINE_POSITION_KEYWORD:
+                  metadata[METADATA_UNDERLINE_POSITION].value = get_integer(&s);
+                  break;
+              case UNDERLINE_THICKNESS_KEYWORD:
+                  metadata[METADATA_UNDERLINE_THICKNESS].value = get_integer(&s);
+                  break;
+              case CAP_HEIGHT_KEYWORD:
+                  metadata[METADATA_CAP_HEIGHT].value = get_integer(&s);
+                  break;
+              case X_HEIGHT_KEYWORD:
+                  metadata[METADATA_X_HEIGHT].value = get_integer(&s);
+                  break;
+              case FONT_ASCENT_KEYWORD:
+                  metadata[METADATA_FONT_ASCENT].value = get_integer(&s);
+                  break;
+              case FONT_DESCENT_KEYWORD:
+                  metadata[METADATA_FONT_DESCENT].value = get_integer(&s);
+                  break;
+              case DEFAULT_CHAR_KEYWORD:
+                  metadata[METADATA_DEFAULT_CHAR].value = get_integer(&s);
+                  break;
+              case FONTBOUNDINGBOX_KEYWORD:
+		  if(scope != BDF_GLOBAL_SCOPE) goto corrupt_file;
+		  globalw = get_integer(&s);
+		  globalh = get_integer(&s);
+		  globalxoff = get_integer(&s);
+		  globalyoff = get_integer(&s);
+		  font->width = globalw;
+		  font->height = globalh;
+                  globaln = ((globalw+7)/8)*2;
+		  font->charsize = font->height*((font->width+7)/8);
+		  metadata[METADATA_FONTBOUNDINGBOX_X].value = globalw;
+		  metadata[METADATA_FONTBOUNDINGBOX_Y].value = globalh;
+		  metadata[METADATA_FONTBOUNDINGBOX_XOFF].value = globalxoff;
+		  metadata[METADATA_FONTBOUNDINGBOX_YOFF].value = globalyoff;
+                  break;
+              case CHARS_KEYWORD:
+		  scope = BDF_CHARS_SCOPE;
+		  font->length = get_integer(&s);
+		  if(font->length > (unsigned int)-1)
+		  {
+                      char buf[32];
+                      sprintf(buf, "Invalid font length (%d).", font->length);
+		      status_error(buf);
+		      //msgBox("We don't support font length > 512 chars. Sorry!", OK, ERROR);
+		      goto go;
+		  }
+		  font->data_size = font->charsize*font->length;
+		  data = (unsigned char *)malloc(font->data_size);
+		  if(!data) goto memory_error;
+		  memset((void *)data, 0, font->data_size);
+		  font->data = data;
+                  //font->unicode_info_size = (512 * 4);
+                  font->unicode_info_size = (font->length * 4);
+                  font->unicode_info = (unsigned char *)malloc(font->unicode_info_size);
+                  if(!font->unicode_info) goto memory_error;
+                  memset((void *)font->unicode_info, 0, font->unicode_info_size);
+                  unicode_info = (unsigned short *)font->unicode_info;
+                  if(!create_char_info(font)) goto memory_error;
+                  char_info = font->char_info;
+                  break;
+              case STARTCHAR_KEYWORD:
+		  scope = BDF_GLYPH_SCOPE;
 		  skip_spaces(&s);
-		  //printf("0x%s: ", s);
-		  if(strlen(s) < n) goto corrupt_file;
-		  unsigned int line = get_hex(s, n);
-		  unsigned int j;
-		  /* convert nibble count to byte count */
-		  n >>= 1;
-		  for(j = 0; j < n; j++)
-		  {
-		    data[data_index+j] = line & 0xFF;
-		    line >>= 8;
-		  }
-		  n = globaln >> 1;
-		  for( ; j < n; j++)
-		  {
-		    data[data_index+j] = 0;
+		  /* check if it is Unicode-encoded */
+                  unsigned int chr = 0;
+                  int skip = 0;
+                  if(strncmp(s, "uni", 3) == 0)
+                  {
+                      skip = 3;
                   }
-		  data_index += n;
-		  i--;
-		}
-		if(i) goto corrupt_file;
-                /* fill the rest of lines (if any) with zeroes */
-                i = globalh-h;
-                while(i--)
-                {
-		  /* how many nibbles do we have per line? */
-		  char n = ((globalw+7)/8)*2;
-		  n >>= 1;
-		  unsigned int j;
-		  for(j = 0; j < n; j++)
+                  else if(s[0] == 'U' || s[0] == 'u' || s[0] == 'C')
+                  {
+                      if(ishexdigit(s[1]) || s[1] == '+')
+                      {
+                          skip++;
+                          if(s[1] == '+') skip++;
+                      }
+                  }
+                  
+		  if(!skip)
 		  {
-		    data[data_index+j] = 0;
+		    int found = 0;
+                    if(*s >= '0' && *s <= '9')
+                    {
+                        chr = get_hex(s, 4);
+                        if(chr) found = 1;
+                    }
+		    else
+                    {
+                        if(postscript_to_unicode(s, &chr)) found = 1;
+                        else
+                        {
+                            //chr = codepoint_to_unicode(s);
+                            if(strlen(s) == 4)
+                            {
+                                chr = get_hex(s, 4);
+                                if(chr) found = 1;
+                            }
+                        }
+                    }
+                    if(!found)
+                    {
+                        char buf[113+strlen(s)];
+                        strcpy(buf, "Char name '");
+                        strcat(buf, s);
+                        strcat(buf, "' not in Unicode, Adobe\n"
+                                    "Standard Encoding, or Postscript char\n"
+                                    "name lists. I Can't read this encoding!");
+                        msgBox(buf, OK, ERROR);
+                        goto go;
+                    }
 		  }
-		  data_index += n;
-                }
-	  }
+		  else
+                  {
+                      s += skip;
+                      chr = get_hex(s, 4);
+                  }
+		  unicode_info[chars*2] = (unsigned short)chr;
+		  unicode_info[(chars*2)+1] = 0xFFFF;
+                  break;
+              case ENDCHAR_KEYWORD:
+		  scope = BDF_CHARS_SCOPE;
+		  chars++;
+                  break;
+              case ENDFONT_KEYWORD:
+		  scope = BDF_GLOBAL_SCOPE;
+		  break;
+              case BBX_KEYWORD:
+		  if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
+		  w = get_integer(&s);
+		  h = get_integer(&s);
+		  xoff = get_integer(&s);
+		  yoff = get_integer(&s);
+                  if(font->char_info)
+                  {
+                      char_info[chars].BBw = w;
+                      char_info[chars].BBh = h;
+                      char_info[chars].BBXoff = xoff;
+                      char_info[chars].BBYoff = yoff;
+                  }
+                  break;
+              case SWIDTH_KEYWORD:
+		  if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
+                  if(font->char_info)
+                  {
+                      int swx1 = get_integer(&s);
+                      int swy1 = get_integer(&s);
+                      char_info[chars].swidthX = swx1;
+                      char_info[chars].swidthY = swy1;
+                  }
+                  break;
+              case DWIDTH_KEYWORD:
+		  if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
+                  if(font->char_info)
+                  {
+                      int dwx1 = get_integer(&s);
+                      int dwy1 = get_integer(&s);
+                      char_info[chars].dwidthX = dwx1;
+                      char_info[chars].dwidthY = dwy1;
+                  }
+                  break;
+              case ENCODING_KEYWORD:
+                  if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
+                  if(font->char_info)
+                  {
+                      int encoding = get_integer(&s);
+                      char_info[chars].encoding = encoding;
+                  }
+                  break;
+              case BITMAP_KEYWORD:
+		  if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
+		  if(!font->data) goto corrupt_file;
+		  if(!w || !h)
+		  {
+		    if(!globalw || !globalh)
+			  goto corrupt_file;
+		    //w = globalw; h = globalh;
+		  }
+		  i = h;
+		  while(i && (s = strtok(NULL, "\n")))
+		  {
+		    /* how many nibbles do we have per line? */
+		    char n = ((w+7)/8)*2;
+		    skip_spaces(&s);
+		    //printf("0x%s: ", s);
+		    if(strlen(s) < n) goto corrupt_file;
+		    unsigned int line = get_hex(s, n);
+		    unsigned int j;
+		    /* convert nibble count to byte count */
+		    n >>= 1;
+		    for(j = 0; j < n; j++)
+		    {
+		      data[data_index+j] = line & 0xFF;
+		      line >>= 8;
+		    }
+		    n = globaln >> 1;
+		    for( ; j < n; j++)
+		    {
+		      data[data_index+j] = 0;
+                    }
+		    data_index += n;
+		    i--;
+		  }
+		  if(i) goto corrupt_file;
+                  /* fill the rest of lines (if any) with zeroes */
+                  i = globalh-h;
+                  while(i--)
+                  {
+		    /* how many nibbles do we have per line? */
+		    char n = ((globalw+7)/8)*2;
+		    n >>= 1;
+		    unsigned int j;
+		    for(j = 0; j < n; j++)
+		    {
+		      data[data_index+j] = 0;
+		    }
+		    data_index += n;
+                  }
+                  break;
+	    }
 	} while((s = strtok(NULL, "\n")));
 	/* check we got all chars */
 	if(chars != font->length) goto corrupt_file;
@@ -954,7 +1019,7 @@ void bdf_init_module()
   strcpy(bdf_module.mod_name, "bdf");
   bdf_module.max_width = 32;
   bdf_module.max_height = 32;
-  bdf_module.max_length = 512;
+  bdf_module.max_length = 65535;
   bdf_module.create_empty_font = bdf_create_empty_font;
   bdf_module.write_to_file = bdf_write_to_file;
   bdf_module.load_font = bdf_load_font;
