@@ -137,7 +137,8 @@ struct hash_elem {
 extern struct hash_elem hashtab[];
 extern size_t hashtab_sz;
 /* bdf_helper.c */
-unsigned int postscript_to_unicode(char *s);
+//unsigned int postscript_to_unicode(char *s);
+int postscript_to_unicode(char *s, unsigned int *res);
 unsigned int codepoint_to_unicode(char *s);
 
 #endif
