@@ -793,7 +793,7 @@ int get_encodings_table(char *table_data, struct font_s *font)
 	short *st = (short *)table_data;
 	int format = pcf_get_lsbint(*(int *)table_data);
 	int swapbytes = need_swap_bytes(format);
-	int swapbits  = 0; //need_swap_bits (format);
+	//int swapbits  = 0; //need_swap_bits (format);
 	int min_char_or_byte2 = st[2];
 	int max_char_or_byte2 = st[3];
 	int min_byte1 = st[4];
@@ -967,13 +967,15 @@ int pcf_write_to_file(FILE *file, struct font_s *font)
     struct metadata_item_s *meta = (struct metadata_item_s *)font->metadata;
     if(!font->char_info)
     {
-        if(!create_char_info(font)) return 0;
+        if(!create_char_info(font)) return 1;
     }
     pcf_check_font_meta(font);
     struct char_info_s *char_info = font->char_info;
     unsigned char *data = font->data;
     u_int32_t tabcount = 7;
-    struct pcf_toc_entry tables[tabcount];
+    //struct pcf_toc_entry tables[tabcount];
+    struct pcf_toc_entry *tables = malloc(tabcount*sizeof(struct pcf_toc_entry));
+    if(!tables) return 1;
   
     // 0 - write the file header
     res = fwrite(filesig, 1, 4, file);
@@ -1059,6 +1061,16 @@ int pcf_write_to_file(FILE *file, struct font_s *font)
     int min = 0, max = 0;
     for(i = 0; i < font->length; i++)
     {
+        /*
+         * TODO: This is an awful hack. Sometimes the font contains out-of-range
+         *       encodings (e.g. when shrinking a lengthier font, or changing 
+         *       format from BDF to PCF, etc). We solve this by truncating the
+         *       encoding value to the max (otherwise we will write outside the
+         *       encodings array boundaries and will probably SIGSEGV or SIGABRT).
+         *       This needs to be FIXED!
+         */
+        if(char_info[i].encoding >= font->length) char_info[i].encoding = font->length-1;
+
         j = char_info[i].encoding;
         if(j < min) min = j;
         else if(j > max) max = j;
@@ -1098,7 +1110,9 @@ int pcf_write_to_file(FILE *file, struct font_s *font)
     // write the properties table
     res = file_write_lsbint(tables[0].format, file);
     res = file_write_lsbint(nprops, file);
-    char strings[string_size];
+    //char strings[string_size];
+    char *strings = (char *)malloc(string_size);
+    if(!strings) return 1;
     char *sp = strings;
     for(i = 0; i < metadata_table_len; i++)
     {
@@ -1288,6 +1302,7 @@ int pcf_write_to_file(FILE *file, struct font_s *font)
         }
         j = (enc1-min_byte1)*(max_char_or_byte2-min_char_or_byte2+1)+enc2-min_char_or_byte2;
         encodings[j] = i;
+        //printf("j = %d, i = %d, enc_len/2 = %d, min = %d, max = %d\n", j, i, enc_len/2, min, max);
     }
     res = fwrite(encodings, 1, enc_len, file);
 
@@ -1301,6 +1316,8 @@ int pcf_write_to_file(FILE *file, struct font_s *font)
         res = file_write_lsbint(char_info[i].swidthX, file);
     }
 
+    free(strings);
+    free(tables);
     free(encodings);
     if(res == EOF) return 1;
     return 0;
